/****************************************************************************
* Copyright (c) 2008, j. montgomery, ATGi                                   *
* All rights reserved.                                                      *
*                                                                           *
* Redistribution and use in source and binary forms, with or without        *
* modification, are permitted provided that the following conditions        *
* are met:                                                                  *
*                                                                           *
* - Redistributions of source code must retain the above copyright          *
*   notice, this list of conditions and the following disclaimer.           *
*                                                                           *
* - Redistributions in binary form must reproduce the above                 *
*   copyright notice, this list of conditions and the following             *
*   disclaimer in the documentation and/or other materials provided         *
*   with the distribution.                                                  *
*                                                                           *
* - Neither the name of ATGi nor the names of its contributors may          *
*   be used to endorse or promote products derived from this software       *
*   without specific prior written permission.                              *
*                                                                           *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE            *
* COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,       *
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES                  *
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR        *
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        *
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       *
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)             *
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       *
* OF THE POSSIBILITY OF SUCH DAMAGE.                                        *
*****************************************************************************
*/
using System;
using System.Collections.Generic;
using System.Text;

namespace DnSmtp.Message
{
	/// <summary>
	/// A collection of <see cref="EmailAddress"/> objects for use in sending mail messages.
	/// </summary>
	public class EmailAddressList: List<EmailAddress>, ICloneable
	{
		/// <summary>
		/// Creates a new instance of the EmailAddressCollection object.
		/// </summary>
		public EmailAddressList()
		{
		}

		/// <summary>
		/// Adds a new email address to the collection, using the string to create an EmailAddress object.
		/// </summary>
		/// <param name="emailAddress">string to add as an email address.</param>
		/// <remarks>This email address will be validated for conformance when the EmailAddress object is created.</remarks>
		public void Add(string emailAddress)
		{
			this.Add(new EmailAddress(emailAddress));
		}

		#region ICloneable Members

		/// <summary>
		/// Creates a new collection with all addresses of the original.
		/// </summary>
		/// <returns>Cloned copy of the collection.</returns>
		public EmailAddressList Clone()
		{
			// Create a new email address list.
			EmailAddressList clone = (EmailAddressList)this.MemberwiseClone();
			// Copy all addresses from this collection into the new collection.
			// (using the InnerList property bypasses validation.)
			//clone..AddRange(this.InnerList);

			return clone;
		}

		object ICloneable.Clone()
		{
			return this.Clone();
		}

		#endregion
	}
}
