/****************************************************************************
* Copyright (c) 2008, j. montgomery, ATGi                                   *
* All rights reserved.                                                      *
*                                                                           *
* Redistribution and use in source and binary forms, with or without        *
* modification, are permitted provided that the following conditions        *
* are met:                                                                  *
*                                                                           *
* - Redistributions of source code must retain the above copyright          *
*   notice, this list of conditions and the following disclaimer.           *
*                                                                           *
* - Redistributions in binary form must reproduce the above                 *
*   copyright notice, this list of conditions and the following             *
*   disclaimer in the documentation and/or other materials provided         *
*   with the distribution.                                                  *
*                                                                           *
* - Neither the name of ATGi nor the names of its contributors may          *
*   be used to endorse or promote products derived from this software       *
*   without specific prior written permission.                              *
*                                                                           *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE            *
* COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,       *
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES                  *
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR        *
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        *
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       *
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)             *
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       *
* OF THE POSSIBILITY OF SUCH DAMAGE.                                        *
*****************************************************************************
*/
using System;
using System.IO;
using System.Text;

namespace DnSmtp.Message.Encoding
{
	/// <summary>
	/// Summary description for Base64EncodingStrategy.
	/// </summary>
	public class Base64EncodingStrategy : IEncodingStrategy
	{
		protected const int MAX_LINE_CHAR_LENGTH=75;
		private static Base64EncodingStrategy _instance;

		public static Base64EncodingStrategy Instance
		{
			get
			{
				if(_instance == null) 
				{
					_instance = new Base64EncodingStrategy();
				}
				return _instance;
			}
		}

		protected Base64EncodingStrategy()
		{
			
		}

		#region IEncodingStrategy Members
		public string Name 
		{
			get { return "base64"; }
		}

		public virtual string Encode(object messageContent)
		{
			if(messageContent == null)
				throw new ArgumentNullException("messageContent");
			
			string result= string.Empty;

			if(messageContent is Stream) 
			{
				byte[] bytes = new byte[((Stream)messageContent).Length];
				((Stream)messageContent).Read(bytes, 0, (int) ((Stream)messageContent).Length);
				result = Convert.ToBase64String(bytes);
			}
			else if(messageContent is String) 
			{
				result = Convert.ToBase64String(ASCIIEncoding.ASCII.GetBytes((string)messageContent));
			}
			else 
			{
				throw new NotImplementedException("Unable to encode message of type " + messageContent.GetType().FullName);
			}
					
			return MailMessage.FormatStringWidth(result, MAX_LINE_CHAR_LENGTH);
		}
		#endregion
	}
}
