/****************************************************************************
* Copyright (c) 2008, j. montgomery, ATGi                                   *
* All rights reserved.                                                      *
*                                                                           *
* Redistribution and use in source and binary forms, with or without        *
* modification, are permitted provided that the following conditions        *
* are met:                                                                  *
*                                                                           *
* - Redistributions of source code must retain the above copyright          *
*   notice, this list of conditions and the following disclaimer.           *
*                                                                           *
* - Redistributions in binary form must reproduce the above                 *
*   copyright notice, this list of conditions and the following             *
*   disclaimer in the documentation and/or other materials provided         *
*   with the distribution.                                                  *
*                                                                           *
* - Neither the name of ATGi nor the names of its contributors may          *
*   be used to endorse or promote products derived from this software       *
*   without specific prior written permission.                              *
*                                                                           *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE            *
* COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,       *
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES                  *
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR        *
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        *
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       *
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)             *
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       *
* OF THE POSSIBILITY OF SUCH DAMAGE.                                        *
*****************************************************************************
*/
using System;
using System.IO;
using System.Text;

namespace DnSmtp.Message.Encoding
{
	/// <summary>
	/// Summary description for EightBitEncodingStrategy.
	/// </summary>
	public class EightBitEncodingStrategy : IEncodingStrategy
	{
		private static EightBitEncodingStrategy _instance;
		
		public static EightBitEncodingStrategy Instance
		{
			get
			{
				if(_instance == null) 
				{
					_instance = new EightBitEncodingStrategy();
				}
				return _instance;
			}
		}

		protected EightBitEncodingStrategy()
		{

		}

		#region IEncodingStrategy Members

		public string Name
		{
			get
			{
				return "8bit";
			}
		}

		public string Encode(object message)
		{
			// the utf encoder will throw an exception if a character isn't within the utf8 char set.
			UTF8Encoding utf8Encoder = new UTF8Encoding(false, true);
			string result = string.Empty;

			if(message is Stream) 
			{
				byte[] bytes = new byte[((Stream)message).Length];
				((Stream)message).Read(bytes, 0, (int) ((Stream)message).Length);
				result = utf8Encoder.GetString(bytes);
			}
			else if(message is String) 
			{
				byte[] encodedBytes = utf8Encoder.GetBytes((string)message);
				result = utf8Encoder.GetString(encodedBytes);
			}
			else 
			{
				throw new NotImplementedException("Unable to encode message of type " + message.GetType().FullName);
			}

			return result;
		}
		#endregion
	}
}
